### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.publisher.interfaces.browser import IBrowserSkinType

# import local interfaces
from ztfy.skin.interfaces import IDialogMenu, ISkinnable

# import Zope3 packages
from z3c.menu.simple.menu import ContextMenuItem
from zope.component import queryUtility
from zope.interface import implements

# import local packages
from ztfy.utils.traversing import getParent

from ztfy.skin import _


class MenuItem(ContextMenuItem):
    """Default menu item"""

    @property
    def css(self):
        css = getattr(self, 'cssClass', '')
        if self.selected:
            css += ' ' + self.activeCSS
        else:
            css += ' ' + self.inActiveCSS
        if self.title.strip().startswith('::'):
            css += ' submenu'
        return css

    @property
    def selected(self):
        return self.request.getURL().endswith('/' + self.viewURL)


class SkinTargetMenuItem(MenuItem):
    """Customized menu item for specific skin targets"""

    skin = None

    def render(self):
        skinnable = getParent(self.context, ISkinnable)
        if skinnable is None:
            return u''
        skin_name = skinnable.getSkin()
        if skin_name is None:
            return u''
        skin = queryUtility(IBrowserSkinType, skin_name)
        if (skin is self.skin) or skin.extends(self.skin):
            return super(SkinTargetMenuItem, self).render()
        else:
            return u''


class JsMenuItem(MenuItem):
    """Customized menu item with javascript link URL"""

    @property
    def url(self):
        return self.viewURL


class SkinTargetJsMenuItem(JsMenuItem):
    """Customized JS menu item for specific skin targets"""

    skin = None

    def render(self):
        skinnable = getParent(self.context, ISkinnable)
        if skinnable is None:
            return u''
        skin_name = skinnable.getSkin()
        if skin_name is None:
            return u''
        skin = queryUtility(IBrowserSkinType, skin_name)
        if (skin is self.skin) or skin.extends(self.skin):
            return super(SkinTargetJsMenuItem, self).render()
        else:
            return u''


class DialogMenuItem(JsMenuItem):
    """Customized javascript menu item used to open a dialog"""

    implements(IDialogMenu)

    target = None

    def render(self):
        result = super(DialogMenuItem, self).render()
        if result and (self.target is not None):
            for resource in self.target.resources:
                resource.need()
        return result


class SkinTargetDialogMenuItem(SkinTargetJsMenuItem):
    """Customized JS dialog menu item for specific skin targets"""

    implements(IDialogMenu)

    target = None

    def render(self):
        result = super(SkinTargetJsMenuItem, self).render()
        if result and (self.target is not None):
            for resource in self.target.resources:
                resource.need()
        return result


class PropertiesMenuItem(MenuItem):
    """Default properties menu item"""

    title = _("Properties")



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2013 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages
import copy

# import Zope3 interfaces
from hurry.query.interfaces import IQuery
from z3c.json.interfaces import IJSONWriter
from z3c.language.switch.interfaces import II18n
from zope.dublincore.interfaces import IZopeDublinCore
from zope.intid.interfaces import IIntIds
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces.browser import IBrowserSkinType
from zope.traversing.interfaces import TraversalError

# import local interfaces
from ztfy.skin.interfaces import ISkinnable, IPresentationTarget, IPresentationForm, IBaseIndexView

# import Zope3 packages
from z3c.form import field
from z3c.formjs import ajax
from z3c.template.template import getLayoutTemplate
from zope.component import queryUtility, queryMultiAdapter, getUtility
from zope.interface import implements
from zope.publisher.skinnable import applySkin
from zope.traversing import namespace

# import local packages
from ztfy.skin.form import DialogEditForm
from ztfy.skin.page import TemplateBasedPage
from ztfy.utils.catalog.index import Text
from ztfy.utils.traversing import getParent


#
# Presentation management classes
#

class PresentationNamespaceTraverser(namespace.view):
    """++presentation++ namespace"""

    def getSkin(self):
        skinnable = getParent(self.context, ISkinnable)
        if skinnable is None:
            return None
        skin_name = skinnable.getSkin()
        if skin_name is None:
            return None
        return queryUtility(IBrowserSkinType, skin_name)

    def traverse(self, name, ignored):
        skin = self.getSkin()
        if skin is None:
            raise TraversalError('++presentation++')
        fake = copy.copy(self.request)
        applySkin(fake, skin)
        adapter = queryMultiAdapter((self.context, fake), IPresentationTarget)
        if adapter is not None:
            try:
                return adapter.target_interface(self.context)
            except:
                pass
        raise TraversalError('++presentation++')


class BasePresentationEditForm(DialogEditForm):
    """Presentation edit form"""

    implements(IPresentationForm)

    layout = getLayoutTemplate()

    def __init__(self, context, request):
        super(BasePresentationEditForm, self).__init__(context, request)
        skin = self.getSkin()
        if skin is None:
            raise NotFound(self.context, self.name, self.request)
        fake = copy.copy(self.request)
        applySkin(fake, skin)
        adapter = queryMultiAdapter((self.context, fake, self), IPresentationTarget)
        if adapter is None:
            adapter = queryMultiAdapter((self.context, fake), IPresentationTarget)
        if adapter is None:
            raise NotFound(self.context, self.name, self.request)
        self.interface = adapter.target_interface

    def getContent(self):
        return self.interface(self.context)

    @property
    def fields(self):
        return field.Fields(self.interface)

    def getSkin(self):
        skinnable = getParent(self.context, ISkinnable)
        if skinnable is None:
            return None
        skin_name = skinnable.getSkin()
        if skin_name is None:
            return None
        return queryUtility(IBrowserSkinType, skin_name)

    @ajax.handler
    def ajaxEdit(self):
        return super(BasePresentationEditForm, self).ajaxEdit(self)

    @ajax.handler
    def ajaxSearch(self):
        writer = getUtility(IJSONWriter)
        title = self.request.form.get('title')
        if not title:
            return writer.write(None)
        query = getUtility(IQuery)
        intids = getUtility(IIntIds)
        result = []
        for obj in query.searchResults(Text(('Catalog', 'title'),
                                            {'query': title + '*',
                                             'ranking': True})):
            i18n = II18n(obj, None)
            result.append({'oid': intids.register(obj),
                           'title': i18n.queryAttribute('title') if i18n is not None else IZopeDublinCore(obj).title})
        return writer.write(result)


#
# Base index view
#

class BaseIndexView(TemplateBasedPage):
    """Base index view"""

    implements(IBaseIndexView)

    presentation = None

    def update(self):
        super(BaseIndexView, self).update()
        adapter = queryMultiAdapter((self.context, self.request, self), IPresentationTarget)
        if adapter is None:
            adapter = queryMultiAdapter((self.context, self.request), IPresentationTarget)
        if adapter is not None:
            interface = adapter.target_interface
            self.presentation = interface(self.context)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2013 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import IWidget, IBoolTerms
from zope.schema.interfaces import IBool

# import local interfaces
from ztfy.skin.layer import IZTFYBrowserLayer

# import Zope3 packages
from z3c.form.term import BoolTerms as BaseBoolTerms
from zope.component import adapts
from zope.interface import implementsOnly, Interface

# import local packages

from ztfy.skin import _


class BoolTerms(BaseBoolTerms):
    """Default yes and no terms are used by default for IBool fields."""

    adapts(Interface, IZTFYBrowserLayer, Interface, IBool, IWidget)
    implementsOnly(IBoolTerms)

    trueLabel = _('yes')
    falseLabel = _('no')



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import ITextWidget, ITextAreaWidget

# import local interfaces
from z3c.form import button
from z3c.formjs import jsaction
# most interfaces have moved to ZTFY.baseskin package; please use them directly...
# following imports are kept only for compatibility reasons
from ztfy.baseskin.interfaces import *
from ztfy.baseskin.interfaces.form import *

# import Zope3 packages
from zope.interface import Attribute, Interface
from zope.schema import Bool, TextLine

# import local packages

from ztfy.skin import _


#
# Default forms buttons interfaces
#

class IAddFormButtons(Interface):
    """Default add form buttons"""

    add = button.Button(name='add', title=_("Add"), condition=checkSubmitButton)


class IDialogAddFormButtons(Interface):
    """Default dialog add form buttons"""

    add = jsaction.JSButton(name='add', title=_("Add"))
    cancel = jsaction.JSButton(name='cancel', title=_("Cancel"))


class IDialogDisplayFormButtons(Interface):
    """Default dialog display form buttons"""

    dialog_close = jsaction.JSButton(name='close', title=_("Close"))


class IEditFormButtons(Interface):
    """Default edit form buttons"""

    submit = button.Button(name='submit', title=_("Submit"), condition=checkSubmitButton)
    reset = button.Button(name='reset', title=_("Reset"))


class IDialogEditFormButtons(Interface):
    """Default dialog edit form buttons"""

    dialog_submit = jsaction.JSButton(name='dialog_submit', title=_("Submit"), condition=checkSubmitButton)
    dialog_cancel = jsaction.JSButton(name='dialog_cancel', title=_("Cancel"))


#
# Custom widgets interfaces
#

class IDateWidget(ITextWidget):
    """Marker interface for date widget"""


class IDatetimeWidget(ITextWidget):
    """Marker interface for datetime widget"""


class IFixedWidthTextAreaWidget(ITextAreaWidget):
    """Marker interface for fixed width text area widget"""


#
# Default views interfaces
#

class IPropertiesMenuTarget(Interface):
    """Marker interface for properties menus"""


class IDialogMenu(Interface):
    """Dialog access menu interface"""

    target = Attribute(_("Target dialog class"))


#
# Breadcrumb interfaces
#

class IBreadcrumbInfo(Interface):
    """Get custom breadcrumb info of a given context"""

    visible = Bool(title=_("Visible ?"),
                   required=True,
                   default=True)

    title = TextLine(title=_("Title"),
                     required=True)

    path = TextLine(title=_("Path"),
                    required=False)


#
# Back-office custom properties marker interface
#

class ICustomBackOfficeInfoTarget(Interface):
    """Marker interface for custom back-office properties"""

    back_interface = Attribute(_("Back-office properties custom interface"))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#
##############################################################################


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.skin.viewlets.custom.interfaces import ICustomViewletManagerHandler, \
                                                 ICustomViewletManager

# import Zope3 packages
from zope.component import getAdapters
from zope.interface import implements
from zope.viewlet.manager import WeightOrderedViewletManager

# import local packages
from ztfy.utils.property import cached_property


class CustomViewletManagerHandler(WeightOrderedViewletManager):
    """Custom viewlet manager based on adapters"""

    implements(ICustomViewletManagerHandler)

    @cached_property
    def managers(self):
        adapters = getAdapters((self.context, self.request, self.__parent__), ICustomViewletManager)
        return sorted((adapter for _name, adapter in adapters),
                      key=lambda x: x.weight)

    def update(self):
        [ manager.update() for manager in self.managers ]

    def render(self):
        return '\n'.join((manager.render() for manager in self.managers))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.skin.interfaces import ICustomBackOfficeInfoTarget
from ztfy.skin.layer import IZTFYBackLayer

# import Zope3 packages
from zope.security.proxy import removeSecurityProxy
from zope.traversing.browser.absoluteurl import absoluteURL

# import local packages
from ztfy.baseskin.metas import *
from ztfy.utils.traversing import getParent


class BaseContentMetasHeadersBackAdapter(object):
    """Base content back-office metas adapter"""

    adapts(Interface, IZTFYBackLayer)
    implements(IContentMetasHeaders)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def metas(self):
        result = []
        back_target = getParent(self.context, ICustomBackOfficeInfoTarget)
        if back_target is not None:
            back_info = removeSecurityProxy(back_target.back_interface)(back_target)
            if getattr(back_info, 'custom_icon', None):
                result.append(LinkMeta('icon', back_info.custom_icon.contentType,
                                       absoluteURL(back_info.custom_icon, self.request)))
            else:
                result.append(LinkMeta('icon', 'image/png', '/@@/favicon.ico'))
        else:
            result.append(LinkMeta('icon', 'image/png', '/@@/favicon.ico'))
        return result



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from hurry.query.interfaces import IQuery
from z3c.language.switch.interfaces import II18n
from zope.intid.interfaces import IIntIds

# import local interfaces
from ztfy.base.interfaces import IBaseContent
from ztfy.base.interfaces.container import IOrderedContainer
from ztfy.skin.interfaces import IDefaultView, IContainedDefaultView
from ztfy.skin.layer import IZTFYBrowserLayer, IZTFYBackLayer

# import Zope3 packages
from z3c.jsonrpc.publisher import MethodPublisher
from zope.component import adapts, getUtility
from zope.interface import implements, Interface
from zope.traversing.browser import absoluteURL

# import local packages
from ztfy.utils.catalog.index import Text


class BaseContentDefaultViewAdapter(object):
    """Default front-office URL adapter"""

    adapts(IBaseContent, IZTFYBrowserLayer, Interface)
    implements(IDefaultView)

    viewname = ''

    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.view = view

    def getAbsoluteURL(self):
        return absoluteURL(self.context, self.request)


class BaseContentDefaultBackViewAdapter(object):
    """Default back-office URL adapter"""

    adapts(IBaseContent, IZTFYBackLayer, Interface)
    implements(IDefaultView)

    viewname = '@@properties.html'

    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.view = view

    def getAbsoluteURL(self):
        return '%s/%s' % (absoluteURL(self.context, self.request), self.viewname)


class BaseContainedDefaultBackViewAdapter(object):
    """Default container back-office URL adapter"""

    adapts(IOrderedContainer, IZTFYBackLayer, Interface)
    implements(IContainedDefaultView)

    viewname = '@@contents.html'

    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.view = view

    def getAbsoluteURL(self):
        return '%s/%s' % (absoluteURL(self.context, self.request), self.viewname)


class BaseContentSearchView(MethodPublisher):
    """Base content XML-RPC search view"""

    def searchByTitle(self, query):
        if not query:
            return []
        query_util = getUtility(IQuery)
        intids = getUtility(IIntIds)
        result = []
        for obj in query_util.searchResults(Text(('Catalog', 'title'),
                                                 {'query': query + '*',
                                                  'ranking': True})):
            result.append({'value': str(intids.register(obj)),
                           'caption': II18n(obj).queryAttribute('title')})
        return result
